#!/usr/bin/perl -w
#
# reporter.cgi is a script which should render an html user interface.
# Copyright (C) 2002 "Freddy Frouin" <freddy.frouin@fluxus.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
################################################################################

use strict;
use RRDs;

my $DEBUG=0;

my $scr_archive_path="/opt/scr/var";

&html_header();
&html_begin();

my %instructions;
if ($ENV{'QUERY_STRING'} eq "") {
  &html_main;
} else {
  my @dum=split(/\&/,$ENV{'QUERY_STRING'});
  foreach (@dum) {
    unless (/^([^=]+)=([^=]+)$/) {
# script manipulation
    } else {
      $instructions{$1}=$2;
    }
  }
}

if (defined($instructions{PAGE})) {
  if ($instructions{PAGE} eq "ARCHIVES") {
    if (defined($instructions{ACTION})) {
      if ($instructions{ACTION} eq "GOTO") {
        html_archives("main");
      } elsif ($instructions{ACTION} eq "EXPAND") {
        html_archives("expand_node","$instructions{NODE}");
      } elsif ($instructions{ACTION} eq "VIEW") {
        html_fileview($scr_archive_path,$instructions{NODE},$instructions{FILE});
      } else {
# script manipulation
      }
    } else {
    }
  } elsif ($instructions{PAGE} eq "REPORTS") {
    if (defined($instructions{ACTION})) {
      if ($instructions{ACTION} eq "GOTO") {
        html_reports("main");
      } elsif ($instructions{ACTION} eq "EXPAND") {
        html_reports("expand_node","$instructions{NODE}");
      } else {
# script manipulation
      }
    } else {
    }
  } else {
# script manipulation
  }
} else {
# script manipulation
}

if ($DEBUG) {
  print "<BR>Instructions</BR>\n";
  foreach (keys %instructions) {
    print "<B>$_<B> $instructions{$_}<BR>\n";
  }

  print "<BR><BR>\n";
  foreach (keys(%ENV)) {
    print "<B>$_</B> $ENV{$_}<BR>\n";
  }
}

html_end();

sub html_reports {
  my $web_report_path="/opt/scr/web/reports";

  my ($action,$node)=(undef,undef);

  unless(defined($_[1])) {
    $action=$_[0];
  } else {
    ($action,$node)=($_[0],$_[1]);
  }
 
  unless(opendir(DIR,"$web_report_path")) {
    html_error("Unable to enter in directory \"$web_report_path\".<BR>Please check rights on that directory.<BR>");
  }
  print "<IMG src=\"/images/reports.gif\" border=\"0\"><BR><BR>\n";
  foreach (readdir(DIR)) {
    next if (/^\.{1,2}/);
    next if (-f "$web_report_path/$_");
    if ($action eq "expand_node" && $_ eq "$node") {
      unless(opendir(SUBDIR,"$web_report_path/$node")) {
        html_error("Unable to enter in directory \"$web_report_path/$node\".<BR>Please check rights on that directory.<BR>");
      }
      print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B><FONT face=\"terminal\">$node</FONT></B><BR>\n";
      my $fn;
      foreach (readdir(SUBDIR)) {
        next if (/^\.{1,2}/);
        $fn=$_;
        $fn=~s/^(.+)\.cgi$/$1/;
        print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/reports/$node/$_\" target=\"main\"><FONT face=\"terminal\"><B>$fn</B></FONT></A><BR>";
      }
      close(SUBDIR);
    } else {
      print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=REPORTS&ACTION=EXPAND&NODE=$_\"><FONT face=\"terminal\"><B>$_</B></FONT></A><BR>\n";
    }
  }
  close(DIR);
  print "<BR><A href=\"/cgi-bin/reporter.cgi\"><IMG src=\"/images/back.gif\" border=\"0\"></A>\n";
}

sub html_fileview {
  unless($#_ == 1) {
# script manipulation
  }
  my ($path,$node,$file)=($_[0],$_[1],$_[2]);

  if ($file=~/\.rrd$/) {
    my $info = RRDs::info("$path/$node/$file");
    unless(defined($$info{filename})) {
      html_error("File \"$path/$node/$file\" do not look like an rrd archives...<BR>May be corrupted.<BR>");
    }
    print "<H4>RRD File $file</H4>\n";
    print "<TABLE><FONT face=\"terminal\">\n";
    print "<TR><TD bgcolor=\"#b7b7b7\"><B>File PATH:<B></TD><TD bgcolor=\"#d6d4d4\"><I>$$info{filename}</I></TD></TR>\n";
    print "<TR><TD bgcolor=\"#b7b7b7\"><B>Last Update:</B></TD><TD bgcolor=\"#d6d4d4\"><I>".localtime($$info{last_update})."</I></TD></TR>\n";
    print "<TR><TD bgcolor=\"#b7b7b7\"><B>Step:<B></TD><TD bgcolor=\"#d6d4d4\"><I>$$info{step}</I></TD></TR>\n";
    print "<TR><TD><BR><BR></TD></TR>\n";

    foreach (sort(keys %$info)) {
      if (/^ds\[([^\]]+)\]\.([^:]+)$/) {
        print "<TR><TD bgcolor=\"#86d183\"><B>Datasource $1</B></TD><TD bgcolor=\"#a3f7a0\">$2</TD><TD bgcolor=\"#a3f7a0\"><I>$$info{$_}</I></TD></TR>\n";
      } elsif (/^rra\[(\d+)\]\.([^\.]+)$/) {
        print "<TR><TD bgcolor=\"#e06662\"><B>RRA $1</B></TD><TD bgcolor=\"#f9a29f\">$2</TD><TD bgcolor=\"#f9a29f\">$$info{$_}</TD></TR>\n";
      } elsif (/^rra\[(\d+)\]\.([^\.]+)\.([^\.]+)$/) {
         print "<TR><TD bgcolor=\"#e06662\"><B>RRA $1</B></TD><TD bgcolor=\"#f9a29f\">$2</TD><TD bgcolor=\"#f9a29f\">$3</TD><TD bgcolor=\"#f9a29f\"><I>$$info{$_}</I></TD></TR>\n";
      }
    }
    print "</FONT></TABLE>\n";
  } elsif ($file=~/\.html$/) {
    print "<H4>HTML File $file</H4>\n";
    unless(open(FD,"$path/$node/$file")) {
      html_error("File \"$path/$node/$file\" could not be opened for reading...<BR>Please check file rights.");
    }
    foreach (<FD>) {
      print $_;
    }
    close(FD);
  } else {
      html_error("This kind of file \"$path/$node/$file\" will not be opened for reading...<BR>.");
  }
}

sub html_archives {
  my ($section,$node)=(undef,undef);

  unless (defined($_[1])) {
    $section=$_[0];
  } else {
    ($section,$node)=($_[0],$_[1]);
  }

  unless(opendir(DIR,$scr_archive_path)) {
    html_error("Unable to enter in directory \"$scr_archive_path\".<BR>Please check server web user rights on that directory.");
  }
  print "<IMG src=\"/images/archives.gif\" border=\"0\"><BR><BR>\n";
  foreach (readdir(DIR)) {
    next if (/^\.{1,2}$/);
    next if (-f "$src_archive_path/$_");
    if ($section eq "expand_node") {
      if (/^$node$/) {
        print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<FONT face=\"terminal\"><B>$node</B></FONT><BR>\n";
        unless(opendir(SUBDIR,"$scr_archive_path/$node")) {
          html_error("Unable to enter in directory \"$scr_archive_path/$node\".<BR>Please check server web user rights on that directory.");
        }
        print "<TABLE>\n";
        my $fn;
        foreach (readdir(SUBDIR)) {
          next if (/^\.{1,2}/);
          if (-f "$scr_archive_path/$node/$_") {
            if (/\.rrd$/) {
              $fn=$_;
              $fn=~s/^(.+)\.rrd$/$1/;
              print "<TR><TD>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=VIEW&NODE=$node&FILE=$_\" target=\"main\"><FONT face=\"terminal\"><B>$fn</B></FONT></A></TD><TD><IMG src=\"/images/rrd.gif\"></TD></TR>\n";
            } elsif (/\.html$/) {
              $fn=$_;
              $fn=~s/^(.+)\.html$/$1/;
              print "<TR><TD>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=VIEW&NODE=$node&FILE=$_\" target=\"main\"><FONT face=\"terminal\"><B>$fn</B></FONT></A></TD><TD><IMG src=\"/images/html.gif\"></TD></TR>\n";
            } else {
              print "<TR><TD>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=VIEW&NODE=$node&FILE=$_\" target=\"main\"><FONT face=\"terminal\"><B>$_</B></FONT></A></TD><TD><IMG src=\"/images/err.gif\"></TD></TR>\n";
            }
          }
        }
        print "</TABLE>\n";
        close(SUBDIR);
      } else {
        print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=EXPAND&NODE=$_\"><FONT face=\"terminal\"><B>$_</B></FONT></A><BR>\n";
      }
    } else {
      print "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=EXPAND&NODE=$_\"><FONT face=\"terminal\"><B>$_</B></FONT></A><BR>\n";
    }
  }
  close(DIR);
  print "<BR><A href=\"/cgi-bin/reporter.cgi\"><IMG src=\"/images/back.gif\" border=\"0\"></A>\n";
}

sub html_main {
  print "<A href=\"/cgi-bin/reporter.cgi?PAGE=ARCHIVES&ACTION=GOTO\"><IMG src=\"/images/archives.gif\" border=\"0\"></A><BR>\n";
  print "<A href=\"/cgi-bin/reporter.cgi?PAGE=REPORTS&ACTION=GOTO\"><IMG src=\"/images/reports.gif\" border=\"0\"></A><BR>\n";
}

sub html_error {
  print "<HTML><HEAD><TITLE>SCR Reporter Error</TITLE></HEAD>\n";
  print "<BODY bgcolor=\"#FFFFFF\">\n";
  print "<H3><I>SCR Error Notification</I></H3>\n";
  print "<P>$_[0]\n";
  print "</BODY></HTML>\n";
  exit 1;
}

sub html_end {
  print "</BODY></HTML>\n";
}

sub html_begin {
  print "<HTML><HEAD><TITLE>Service Check Reporter - Web Services</TITLE></HEAD>";
  print "<BODY bgcolor=\"#FFFFFF\">\n";
}

sub html_header {
  print "Content-type: text/html\n";
  print "\n";
}
